{ **************************************************************************** }
{ *                                                                          * }
{ * CACCIA AL LADRO (VTech, Mini Time & Fun) Simulator                       * }
{ * by Luca "MADrigal" Antignano, March 2008 (revision: August 2021)         * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;    im_background_screen_top: TImage;

    btn_game_a_down:          TImage;    btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;    btn_game_b_top:           TImage;
    btn_time_down:            TImage;    btn_time_top:             TImage;
                                         btn_set_top:              TImage;

    btn_left_down:            TImage;    btn_left_top:             TImage;
    btn_right_down:           TImage;    btn_right_top:            TImage;
    btn_shoot_left_down:      TImage;    btn_shoot_left_top:       TImage;
    btn_shoot_right_down:     TImage;    btn_shoot_right_top:      TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_1: TImage; im_panel_time_2: TImage; im_panel_alarm: TImage; im_panel_date: TImage; im_panel_chrono: TImage;

    im_panel_sound: TImage; im_panel_bell: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; im_panel_digit_pos_5: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    im_panel_number_small_0: TImage; im_panel_number_small_1: TImage; im_panel_number_small_2: TImage; im_panel_number_small_3: TImage; im_panel_number_small_4: TImage;
    im_panel_number_small_5: TImage; im_panel_number_small_6: TImage; im_panel_number_small_7: TImage; im_panel_number_small_8: TImage; im_panel_number_small_9: TImage;

    // on-screen items
    im_hero_1: TImage;          im_hero_2: TImage;          im_hero_3: TImage;

    im_enemy_1_1: TImage;       im_enemy_2_1: TImage;       im_enemy_3_1: TImage;
    im_enemy_1_2: TImage;       im_enemy_2_2: TImage;       im_enemy_3_2: TImage;
    im_enemy_1_3: TImage;       im_enemy_2_3: TImage;       im_enemy_3_3: TImage;

    im_enemy_helper_1: TImage;  im_enemy_helper_2: TImage;  im_enemy_helper_3: TImage;

    im_shot_1_4: TImage;        im_shot_2_4: TImage;        im_shot_3_4: TImage;
    im_shot_1_3: TImage;        im_shot_2_3: TImage;        im_shot_3_3: TImage;
    im_shot_1_2: TImage;        im_shot_2_2: TImage;        im_shot_3_2: TImage;
    im_shot_1_1: TImage;        im_shot_2_1: TImage;        im_shot_3_1: TImage;

    im_saveme_1_3: TImage;      im_saveme_2_3: TImage;      im_saveme_3_3: TImage;
    im_saveme_1_2: TImage;      im_saveme_2_2: TImage;      im_saveme_3_2: TImage;
    im_saveme_1_1: TImage;      im_saveme_2_1: TImage;      im_saveme_3_1: TImage;

    im_miss_1: TImage; im_miss_2: TImage; im_miss_3: TImage; im_miss_4: TImage; im_miss_5: TImage;

{ * TIMERS                                                                   * }
    Timer_Bonus:        TTimer;
    Timer_Clock:        TTimer;
    Timer_Date:         TTimer;
    Timer_Enemy_Attack: TTimer;
    Timer_Enemy_Helper: TTimer;
    Timer_Enemy_Move:   TTimer;
    Timer_Game_Over:    TTimer;
    Timer_Game_Start:   TTimer;
    Timer_Hit:          TTimer;
    Timer_Miss:         TTimer;
    Timer_Shot:         TTimer;
    Timer_Startup:      TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                 (Sender: TObject);
    procedure FormActivate               (Sender: TObject);
    procedure FormClose                  (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All               (b: boolean);

    procedure Vid_Show_All_Enemy         (b: boolean);
    procedure Vid_Show_All_Enemy_Helper  (b: boolean);
    procedure Vid_Show_All_Hero          (b: boolean);
    procedure Vid_Show_All_Miss          (b: boolean);
    procedure Vid_Show_All_Saveme        (b: boolean);
    procedure Vid_Show_All_Shot          (b: boolean);

    procedure Vid_Show_Enemy             (p: integer);
    procedure Vid_Show_Enemy_Helper;
    procedure Vid_Show_Hero              (var p: integer; b: boolean);
    procedure Vid_Show_Saveme            (p: integer);

    procedure Vid_Show_Panel_Score       (var p: integer; b: boolean);
    procedure Vid_Show_Panel_Miss        (var p: integer);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                   (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove     (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_shoot_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_shoot_left_topMouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_shoot_right_topMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_shoot_right_topMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    
    procedure FormKeyDown                (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                  (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;           procedure Inp_game_a_up;
    procedure Inp_game_b_down;           procedure Inp_game_b_up;
    procedure Inp_time_down;             procedure Inp_time_up;

    procedure Inp_left_down;             procedure Inp_left_up;
    procedure Inp_right_down;            procedure Inp_right_up;
    procedure Inp_shoot_down;         // procedure Inp_shoot_up

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Calculate_Speed;
    procedure Gam_Do_Miss;
    procedure Gam_Game_Start;
    procedure Gam_Generate_Enemy;
    procedure Gam_Move_Enemy_Helper;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_BonusTimer           (Sender: TObject);
    procedure Timer_ClockTimer           (Sender: TObject);
    procedure Timer_DateTimer            (Sender: TObject);
    procedure Timer_Enemy_AttackTimer    (Sender: TObject);
    procedure Timer_Enemy_HelperTimer    (Sender: TObject);
    procedure Timer_Enemy_MoveTimer      (Sender: TObject);
    procedure Timer_Game_OverTimer       (Sender: TObject);
    procedure Timer_Game_StartTimer      (Sender: TObject);
    procedure Timer_HitTimer             (Sender: TObject);
    procedure Timer_MissTimer            (Sender: TObject);
    procedure Timer_ShotTimer            (Sender: TObject);
    procedure Timer_StartupTimer         (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup             (Sender: TObject);
    procedure pm_1_aboutClick            (Sender: TObject);
    procedure pm_2_readmeClick           (Sender: TObject);
    procedure pm_3_whatsnewClick         (Sender: TObject);
    procedure pm_4_manualClick           (Sender: TObject);
    procedure pm_6_configClick           (Sender: TObject);
    procedure pm_7_keysClick             (Sender: TObject);
    procedure pm_8_hiscoreClick          (Sender: TObject);
  {}procedure pm_10_fullscreenClick      (Sender: TObject);
  {}procedure pm_11_splashscreenClick    (Sender: TObject);
{}//procedure pm_12_controllerClick      (Sender: TObject);
  {}procedure pm_14_exitClick            (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick     (Sender: TObject);

    procedure la_menu_itemMouseEnter     (Sender: TObject);
    procedure la_menu_itemMouseLeave     (Sender: TObject);
    procedure la_menu_itemMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu      (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Caccia al Ladro (VTech, Mini Time & Fun)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\CacciaL50';                                  // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A';
  HINT_GAME_B = 'to start game B';
  HINT_TIME   = 'to switch mode';
  HINT_SET    = 'button disabled';
  HINT_MOVE   = 'arrow keys';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_enemy,
  pfs_gameover,
  pfs_miss,
  pfs_shot,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound,                                                                       // sound enabled? (depending on selected driver on splash screen)
  bSoundIngame:            boolean;                                             // sound enabled? (depending on selection when starting game)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..5]       of TImage;                         // empty sprites to host score/time numbers
  aIm_panel_number,
  aIm_panel_number_small:  array[0..9]       of TImage;                         // numbers for score/time
  aIm_panel_miss:          array[1..3]       of TImage;                         // misses counter

  aIm_hero:                array[1..3]       of TImage;                         // main character sprites
  aIm_saveme:              array[1..3, 1..3] of TImage;                         // sprites of items to save from enemy attacks
  aIm_enemy:               array[1..3, 1..3] of TImage;                         // enemy sprites
  aIm_enemy_helper:        array[1..3]       of TImage;                         // sprites of character who helps enemy
  aIm_shot:                array[1..3, 1..4] of TImage;                         // shots from main character to enemy
  aIm_miss:                array[1..5]       of TImage;                         // misses counter sprites

{ * arrays of numbers * }
  aSavemeValue:            array[1..3]       of integer;                        // how many pieces of item to save (tooth)

  rGameLevels:             array[8..9]       of record                          // settings for Game A and B (8: game A; 9: game B)
                                                  iShotSpeed  : integer;        // speed of bullet shot by main character
                                                  iEnemySpeed : array[0..9] of integer; // speed of enemy
                                                end;

{ * arrays of mixed types * }
  aKeys:                   array[1..7]        of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44]       of string;                        // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-9999
  iHiScoreA,                                                                    // 0-9999
  iHiScoreB,                                                                    // 0-9999
  iMiss,                                                                        // 0-3

  iCharPos,                                                                     // 1-3
  iShotPos,                                                                     // 1-3
  iEnemyPos,                                                                    // 1-3
  iHelperPos                                                                    // 1-3
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bKeyPressed,                                                                  // true while player keeps any key pressed
  bGameAPressed,                                                                // true while 'Game A' button is being kept pressed
  bGameBPressed,                                                                // true while 'Game B' button is being kept pressed
  bControllers,                                                                 // true while player can actually operate the main character
  bControllersShoot                                                             // true while player can actually shoot
                         : boolean;

  iLoopGameStart,                                                               // used in TimerGameStart
  iLoopHit,                                                                     // used in TimerHit
  iLoopMiss,                                                                    // used in TimerMiss
  iLoopGameOver,                                                                // used in TimerGameOver
  iLoopBonus,                                                                   // used in TimerBonus
  iLoopShot,                                                                    // used in TimerShot
  iLoopEnemy                                                                    // used in TimerEnemyAttack and TimerEnemyMove
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  aIm_panel_number_small[0] := im_panel_number_small_0; aIm_panel_number_small[1] := im_panel_number_small_1; aIm_panel_number_small[2] := im_panel_number_small_2; aIm_panel_number_small[3] := im_panel_number_small_3; aIm_panel_number_small[4] := im_panel_number_small_4;
  aIm_panel_number_small[5] := im_panel_number_small_5; aIm_panel_number_small[6] := im_panel_number_small_6; aIm_panel_number_small[7] := im_panel_number_small_7; aIm_panel_number_small[8] := im_panel_number_small_8; aIm_panel_number_small[9] := im_panel_number_small_9;

  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4; aIm_panel_digit_pos[5] := im_panel_digit_pos_5;

  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;

  // on-screen items
  aIm_hero[1] := im_hero_1;                 aIm_hero[2] := im_hero_2;                 aIm_hero[3] := im_hero_3;

  aIm_saveme[1, 1] := im_saveme_1_3;        aIm_saveme[2, 1] := im_saveme_2_3;        aIm_saveme[3, 1] := im_saveme_3_3;
  aIm_saveme[1, 2] := im_saveme_1_2;        aIm_saveme[2, 2] := im_saveme_2_2;        aIm_saveme[3, 2] := im_saveme_3_2;
  aIm_saveme[1, 3] := im_saveme_1_1;        aIm_saveme[2, 3] := im_saveme_2_1;        aIm_saveme[3, 3] := im_saveme_3_1;

  aIm_enemy[1, 1] := im_enemy_1_1;          aIm_enemy[2, 1] := im_enemy_2_1;          aIm_enemy[3, 1] := im_enemy_3_1;
  aIm_enemy[1, 2] := im_enemy_1_2;          aIm_enemy[2, 2] := im_enemy_2_2;          aIm_enemy[3, 2] := im_enemy_3_2;
  aIm_enemy[1, 3] := im_enemy_1_3;          aIm_enemy[2, 3] := im_enemy_2_3;          aIm_enemy[3, 3] := im_enemy_3_3;

  aIm_enemy_helper[1] := im_enemy_helper_1; aIm_enemy_helper[2] := im_enemy_helper_2; aIm_enemy_helper[3] := im_enemy_helper_3;

  aIm_shot[1, 4] := im_shot_1_4;            aIm_shot[2, 4] := im_shot_2_4;            aIm_shot[3, 4] := im_shot_3_4;
  aIm_shot[1, 3] := im_shot_1_3;            aIm_shot[2, 3] := im_shot_2_3;            aIm_shot[3, 3] := im_shot_3_3;
  aIm_shot[1, 2] := im_shot_1_2;            aIm_shot[2, 2] := im_shot_2_2;            aIm_shot[3, 2] := im_shot_3_2;
  aIm_shot[1, 1] := im_shot_1_1;            aIm_shot[2, 1] := im_shot_2_1;            aIm_shot[3, 1] := im_shot_3_1;

  aIm_miss[1] := im_miss_1; aIm_miss[2] := im_miss_2; aIm_miss[3] := im_miss_3; aIm_miss[4] := im_miss_4; aIm_miss[5] := im_miss_5;

  // arrays init
  with rGameLevels[8] do
    begin
      iShotSpeed := 100;
      iEnemySpeed[0] := 340; iEnemySpeed[1] := 320; iEnemySpeed[2] := 300; iEnemySpeed[3] := 290; iEnemySpeed[4] := 285; iEnemySpeed[5] := 280; iEnemySpeed[6] := 270; iEnemySpeed[7] := 260; iEnemySpeed[8] := 255; iEnemySpeed[9] := 250;
    end;

  with rGameLevels[9] do
    begin
      iShotSpeed := 80;
      iEnemySpeed[0] := 280; iEnemySpeed[1] := 260; iEnemySpeed[2] := 250; iEnemySpeed[3] := 230; iEnemySpeed[4] := 220; iEnemySpeed[5] := 210; iEnemySpeed[6] := 200; iEnemySpeed[7] := 190; iEnemySpeed[8] := 180; iEnemySpeed[9] := 170;
    end;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A"        (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B"        (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"          (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 39);                             // "Left"          (default: "LEFT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 40);                             // "Right"         (default: "RIGHT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 42);                             // "Shoot (left)"  (default: "CONTROL")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 41);                             // "Shoot (right)" (default: "ALT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 7 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint      := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint        := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_set_top.Hint         := HINT_SET;                                                       // "Set"

  btn_left_top.Hint        := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];                     // "Left"
  btn_right_top.Hint       := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Right"
  btn_shoot_left_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Shoot (left)"
  btn_shoot_right_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Shoot (right)"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                        // enables double buffering for rendering of  this form

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bKeyPressed := false;                                                         // no key is being pressed
  bGameAPressed := false;                                                       // 'Game A' button is not currently pressed
  bGameBPressed := false;                                                       // 'Game B' button is not currently pressed
  bControllers := false;                                                        // main character can't move
  bControllersShoot := false;                                                   // main character can't shoot

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_enemy); FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_miss);  FSOUND_Sample_Free(pfs_shot);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides nearly all sprites (except score panel)                       }
  var
    i: integer;
begin
  im_panel_game_a.Visible     := b;
  im_panel_game_b.Visible     := b;
  im_panel_time_1.Visible     := b;
  im_panel_time_2.Visible     := b;
  im_panel_alarm.Visible      := b;
  im_panel_date.Visible       := b;
  im_panel_chrono.Visible     := b;
  im_panel_bell.Visible       := b;
  im_panel_sound.Visible      := b;
  im_panel_time_am.Visible    := b;
  im_panel_time_pm.Visible    := b;
  im_panel_time_colon.Visible := b;

  for i := 1 to 5 do
    aIm_panel_digit_pos[i].Visible := true;

  i := 8888; Vid_Show_Panel_Score(i, b);

  if b then
    aIm_panel_digit_pos[5].Picture := aIm_panel_number_small[8].Picture
  else
    aIm_panel_digit_pos[5].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Enemy(b);
  Vid_Show_All_Enemy_Helper(b);
  Vid_Show_All_Hero(b);
  Vid_Show_All_Miss(b);
  Vid_Show_All_Saveme(b);
  Vid_Show_All_Shot(b);
end;





procedure TForm1.Vid_Show_All_Enemy(b: boolean);
{ displays/hides all enemies                                                   }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_enemy[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Enemy_Helper(b: boolean);
{ displays/hides all enemy helpers                                             }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_enemy_helper[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all main character sprites                                    }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_hero[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Miss(b: boolean);
{ displays/hides all "misses" sprites                                          }
  var
    i: integer;
begin
  for i := 1 to 5 do
    aIm_miss[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Saveme(b: boolean);
{ displays/hides all "friendly" items                                          }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_saveme[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Shot(b: boolean);
{ displays/hides all shots from main character to enemies                      }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 4 do
      aIm_shot[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_Enemy(p: integer);
{ display/hides enemy in 'p' position                                          }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    if i = iEnemyPos then
      begin
        aIm_enemy[i, 1].Visible := true;

        if p = 1 then
          begin
            aIm_enemy[i, 2].Visible := true;
            aIm_enemy[i, 3].Visible := false;
          end
        else
          begin
            aIm_enemy[i, 2].Visible := false;
            aIm_enemy[i, 3].Visible := true;
          end;
      end
    else
      for j := 1 to 3 do
        aIm_enemy[i, j].Visible := false;
end;





procedure TForm1.Vid_Show_Enemy_Helper;
{ displays/hides enemy helper in 'p' position                                  }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_enemy_helper[i].Visible := (i = iHelperPos);
end;





procedure TForm1.Vid_Show_Hero(var p: integer; b: boolean);
{ display/hides main character in 'p' position                                 }
begin
  aIm_hero[p].Visible := b;
end;





procedure TForm1.Vid_Show_Saveme(p: integer);
{ displays/hides "friend" item in 'p' position                                 }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_saveme[p, i].Visible := (i <= aSaveMeValue[p]);
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "...0"
    Dec(t, 10000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                               // display
      if t > 999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // thousands
      if t > 99  then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t > 9   then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture;                                            // units
                      aIm_panel_digit_pos[5].Picture := nil;
    end
  else                                                                                  // hide
    for i := 1 to 5 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays as many sprites as p on the misses counter (0=nothing visible)      }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays as many sprites as p, hides the remaining ones
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound and bSoundIngame then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound and bSoundIngame then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ mode button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ mode button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.btn_shoot_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SHOOT (LEFT) button: PRESS                                                   }
begin
  if Button = mbLeft then
    begin
      btn_shoot_left_down.Visible := true;
      Inp_shoot_down;
    end;
end;





procedure TForm1.btn_shoot_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SHOOT (LEFT) button: RELEASE                                                 }
begin
  if Button = mbLeft then
    begin
      btn_shoot_left_down.Visible := false;
      bKeyPressed := false;
    end;
end;





procedure TForm1.btn_shoot_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SHOOT (RIGHT) button: PRESS                                                  }
begin
  if Button = mbLeft then
    begin
      btn_shoot_right_down.Visible := true;
      Inp_shoot_down;
    end;
end;





procedure TForm1.btn_shoot_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SHOOT (RIGHT)  button: RELEASE                                               }
begin
  if Button = mbLeft then
    begin
      btn_shoot_right_down.Visible := false;
      bKeyPressed := false;
    end;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_left_down   else                              // Left
  if Key = aKeys[5].word then Inp_right_down  else                              // Right
  if Key = aKeys[6].word then begin btn_shoot_left_down.Visible := true; Inp_shoot_down; end else // Shoot (left)
  if Key = aKeys[7].word then begin btn_shoot_right_down.Visible := true; Inp_shoot_down; end;    // Shoot (right)
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_left_up   else                                // Left
  if Key = aKeys[5].word then Inp_right_up  else                                // Right
  if Key = aKeys[6].word then begin btn_shoot_left_down.Visible := false; bKeyPressed := false; end else // Shoot (left)
  if Key = aKeys[7].word then begin btn_shoot_right_down.Visible := false; bKeyPressed := false; end;    // Shoot (right)
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if not bGameAPressed then
    begin
      bGameAPressed := true;

      if (iMode = 0) then                                                       // ACL ->
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 6;                                                         // -> HiScore A
            Gam_Set_Mode;
          end;
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;
  bGameAPressed := false;

  if (iMode = 6) then                                                           // HiScore A ->
    begin
      iMode := 8;                                                               // -> Game A
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if not bGameBPressed then
    begin
      bGameBPressed := true;

      if (iMode = 0) then                                                       // ACL
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 7;                                                         // ->  HiScore B
            Gam_Set_Mode;
          end;
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;
  bGameBPressed := false;

  if (iMode = 7) then                                                           // HiScore B ->
    begin
      iMode := 9;                                                               // -> Game B
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_time_down;
{ time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if ( (iMode >= 0) and (iMode < 6) ) or (iMode = 10) then                  // any mode except hiscore/game modes ->
        begin
          Inc(iMode);                                                           // -> Time 1 -> Time 2 -> Alarm -> Date -> Chrono ->

          if iMode > 5 then
            iMode := 1;

          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_time_up;
{ time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_left_down;
{ left button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if (iMode = 6) or (iMode = 7) then
        begin
          bSoundIngame := false;
          im_panel_sound.Visible := false;
        end;

      if bControllers and (iCharPos > 1) then
        begin
          Vid_Show_Hero(iCharPos, false);                                       // hides main character
          Dec(iCharPos);                                                        // updates coordinate
          Vid_Show_Hero(iCharPos, true);                                        // displays main character
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ left button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ right button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if bControllers and (iCharPos < 3) then
        begin
          Vid_Show_Hero(iCharPos, false);                                       // hides main character
          Inc(iCharPos);                                                        // updates coordinate
          Vid_Show_Hero(iCharPos, true);                                        // displays main character
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ right button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_shoot_down;
{ any fire button: PRESS                                                       }
begin
  if bControllers and (not bKeyPressed) then
    begin
      bKeyPressed := true;

      if bControllersShoot then
        begin
          bControllersShoot := false;
          Snd_Play(pfs_shot);
          aIm_shot[iCharPos, 1].Visible := true;
          iShotPos := iCharPos;

          iLoopShot := 1;
          Timer_Shot.Enabled := true;
        end;
    end;
end;











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Bonus.Enabled        := false;
  Timer_Clock.Enabled        := false;
  Timer_Date.Enabled         := false;
  Timer_Enemy_Attack.Enabled := false;
  Timer_Enemy_Helper.Enabled := false;
  Timer_Enemy_Move.Enabled   := false;
  Timer_Game_Over.Enabled    := false;
  Timer_Game_Start.Enabled   := false;
  Timer_Hit.Enabled          := false;
  Timer_Miss.Enabled         := false;
  Timer_Shot.Enabled         := false;
  Timer_Startup.Enabled      := false;
end;





procedure TForm1.Gam_Calculate_Speed;
{ sets the difficulty level, based on the game mode and current score          }
  var
    i: integer;
begin
  i := iScore;

  while i > 990 do
    Dec(i, 1000);

  Timer_Enemy_Attack.Interval := rGameLevels[iMode].iEnemySpeed[i div 100];
  Timer_Enemy_Move.Interval := Timer_Enemy_Attack.Interval;

  if (iScore / 1000) = (iScore div 1000) then
    begin                                                                       // bonus routine
      Gam_Stop_All_Timers;
      Vid_Show_All_Enemy(false);                                                // makes enemy disappear before he's generated

      bControllers := false;
      bControllersShoot := false;

      iLoopBonus := 1;
      Timer_Bonus.Interval := 750;
      Timer_Bonus.Enabled := true;
    end;
end;





procedure TForm1.Gam_Do_Miss;
{ initialises the "miss" procedure then starts the timer                       }
begin
  Gam_Stop_All_Timers;
  Vid_Show_All_Shot(false);

  bControllers := false;
  bControllersShoot := false;

  iLoopMiss := 1;
  Timer_Miss.Interval := 1;
  Timer_Miss.Enabled := true;
end;





procedure TForm1.Gam_Game_Start;
{ inits all game variables                                                     }
begin
  if bSoundIngame then
    begin
      Timer_Game_Start.Interval := 3000;
      Snd_Play(pfs_start);
    end
  else
    Timer_Game_Start.Interval := 100;

  iLoopGameStart := 1;
  Timer_Game_Start.Enabled := true;
end;





procedure TForm1.Gam_Generate_Enemy;
{ generates a new enemy                                                        }
begin
  iEnemyPos := Random(3) + 1;
  Vid_Show_Enemy(1);
  Snd_Play(pfs_enemy);

  if Random(5) < 2 then
    begin                                                                       // ready to attack
      iLoopEnemy := 1;
      Timer_Enemy_Attack.Enabled := true;
    end
  else
    begin                                                                       // ready to move
      iLoopEnemy := 1;
      Timer_Enemy_Move.Enabled := true;
    end;
end;





procedure TForm1.Gam_Move_Enemy_Helper;
{ moves the enemy helper to the left or right randomly                         }
  var
    i: integer;
begin
  i := iHelperPos;

  repeat
    iHelperPos := Random(3) + 1;
  until
    iHelperPos <> i;

  Vid_Show_Enemy_Helper;

  Timer_Enemy_Helper.Interval := (Random(10) + 5) * 200;
  Timer_Enemy_Helper.Enabled := true;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/etc mode, depending on the value of the iMode variable  }
  var
    i: integer;
begin
  case iMode of
    0: begin                                                                    // ACL
         Vid_Show_All(true);                                                            // game is turned on

         Timer_Startup.Interval := 30000;
         Timer_Startup.Enabled := true;
       end;
    1: begin                                                                    // TIME 1
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_time_1.Visible := true;                                               // displays "time 1"
         im_panel_time_colon.Visible := true;                                           // displays colon

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;                                                   // clock starts ticking...
       end;
    2: begin                                                                    // TIME 2
         im_panel_time_1.Visible := false;                                              // hides "time 1"
         im_panel_time_2.Visible := true;                                               // displays "time 2"
       end;
    3: begin                                                                    // ALARM
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_alarm.Visible := true;                                                // displays "alarm"
         im_panel_time_am.Visible := true;                                              // displays "PM"
         im_panel_time_colon.visible := true;                                           // displays colon
         i := 1200; Vid_Show_Panel_Score(i, true);                                      // 12:00
       end;
    4: begin                                                                    // DATE
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_date.Visible := true;                                                 // displays "date"
         im_panel_time_colon.visible := true;                                           // displays colon

         Timer_Date.Interval := 1;
         Timer_Date.Enabled := true;                                                    // clock starts ticking...
       end;
    5: begin                                                                    // CHRONO
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_chrono.Visible := true;                                               // displays "chrono"
         im_panel_time_colon.Visible := true;                                           // displays colon

         for i := 1 to 4 do
           aIm_panel_digit_pos[i].Picture := aIm_panel_number[0].Picture;               // 00:00 x

         aIm_panel_digit_pos[5].Picture := aIm_panel_number_small[0].Picture;           // xx:xx 0
       end;
    6, 7: begin                                                                 // HISCORE A / B
         Gam_Stop_All_Timers;
         Vid_Show_All(false);

         case iMode of
           6: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         for i := 1 to 2 do
           aIm_miss[i].Visible := true;

         for i := 1 to 3 do
           begin
             aSavemeValue[i] := 3;
             Vid_Show_Saveme(i);
           end;

         iMiss := 0;      Vid_Show_Panel_Miss(iMiss);
         iHelperPos := 2; Vid_Show_Enemy_Helper;
         iCharPos := 2;   Vid_Show_Hero(iCharPos, true);

         bSoundIngame := true; im_panel_sound.Visible := true;
       end;
    8, 9: Gam_Game_Start;                                                       // GAME A / B
   10: begin                                                                    // GAME OVER
         // does nothing...
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_BonusTimer(Sender: TObject);
  var
    i: integer;
begin
  case iLoopBonus of
    1: begin Snd_Play(pfs_shot);  iHelperPos := 1; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := false; aIm_miss[3].Visible := true;  Timer_Bonus.Interval := 500; end;
    2: begin Snd_Play(pfs_enemy); iHelperPos := 2; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := true;  aIm_miss[3].Visible := false; end;
    3: begin Snd_Play(pfs_shot);  iHelperPos := 3; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := false; aIm_miss[3].Visible := true;  end;
    4: begin Snd_Play(pfs_enemy); iHelperPos := 1; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := true;  aIm_miss[3].Visible := false; end;
    5: begin Snd_Play(pfs_shot);  iHelperPos := 2; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := false; aIm_miss[3].Visible := true;  end;
    6: begin Snd_Play(pfs_enemy); iHelperPos := 3; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := true;  aIm_miss[3].Visible := false; end;
    7: begin Snd_Play(pfs_shot);  iHelperPos := 1; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := false; aIm_miss[3].Visible := true;  end;
    8: begin Snd_Play(pfs_enemy); iHelperPos := 2; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := true;  aIm_miss[3].Visible := false; end;
    9: begin Snd_Play(pfs_shot);  iHelperPos := 3; Vid_Show_Enemy_Helper; aIm_miss[2].Visible := false; aIm_miss[3].Visible := true;  end;
   10: begin
         aIm_miss[2].Visible := true;
         aIm_miss[3].Visible := false;

         for i := 1 to 3 do
           begin
             aSavemeValue[i] := 3;
             Vid_Show_Saveme(i);
           end;

         Timer_Enemy_Helper.Interval := (Random(10) + 5) * 200;
         Timer_Enemy_Helper.Enabled := true;

         Gam_Generate_Enemy;
         bControllers := true;
         bControllersShoot := true;

         Timer_Bonus.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopBonus);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min div 10].Picture;       // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min mod 10].Picture;       // minute: units
  aIm_panel_digit_pos[5].Picture := nil;                                        // small digit

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_DateTimer(Sender: TObject);
{ transforms current date into "sprites" form, then displays it                }
  var
    present:          TDateTime;
    day, month, year: Word;
begin
  present := Now;                                                               // gets system date...
  DecodeDate(present, year, month, day);                                        // and decodes it

  if month > 9 then                                                             // month: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[month div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[month mod 10].Picture;     // month: units

  if day > 9 then                                                               // day: tens
    aIm_panel_digit_pos[3].Picture := aIm_panel_number[day div 10].Picture
  else
    aIm_panel_digit_pos[3].Picture := nil;

  aIm_panel_digit_pos[4].Picture := aIm_panel_number[day mod 10].Picture;       // day: units
  aIm_panel_digit_pos[5].Picture := nil;                                        // small digit

  Timer_Date.Interval := 1000;
end;





procedure TForm1.Timer_Enemy_AttackTimer(Sender: TObject);
{ enemy attacks                                                                }
begin
  Inc(iLoopEnemy);

  case iLoopEnemy of
    2: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    3: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
    4: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    5: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
    6: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    7: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
    8: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    9: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
   10: begin Vid_Show_Enemy(2); Dec(aSavemeValue[iEnemyPos]); Vid_Show_Saveme(iEnemyPos); Snd_Play(pfs_enemy); end;
   11: if aSavemeValue[iEnemyPos] = 0 then
         Gam_Do_Miss
       else
         begin
           // iEnemyPos := Random(3) + 1;                                       // removed because enemy stands there after attacking, and moves later
           Vid_Show_Enemy(1);
           Snd_Play(pfs_enemy);

           iLoopEnemy := 1;
           Timer_Enemy_Move.Enabled := true;

           Timer_Enemy_Attack.Enabled := false;
         end;
  end; // case..of
end;





procedure TForm1.Timer_Enemy_HelperTimer(Sender: TObject);
{ enemy helper moves                                                           }
begin
  Gam_Move_Enemy_Helper;

  Timer_Enemy_Helper.Enabled := false;
end;





procedure TForm1.Timer_Enemy_MoveTimer(Sender: TObject);
{ enemy moves                                                                  }
begin
  Inc(iLoopEnemy);

  case iLoopEnemy of
    2: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    3: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
    4: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    5: begin Vid_Show_Enemy(1); Snd_Play(pfs_enemy); end;
    6: begin Vid_Show_Enemy(2); Snd_Play(pfs_enemy); end;
    7: begin
         Timer_Enemy_Move.Enabled := false;

         Gam_Generate_Enemy; // must stay at the END
       end;
  end; // case..of
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game is over: then freeze for 30 seconds, and finally change mode to "Time   }
{ 1", if no button is pressed                                                  }
begin
  case iLoopGameOver of
    1: begin
         if (iMode = 8) and (iScore > iHiScoreA) then
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 9) and (iScore > iHiScoreB) then
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         iMode := 10;

         iLoopGameOver := 2;
         Timer_Game_Over.Interval := 30000;
       end;
    2: begin
         iMode := 1;
         Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ start tune ends, game actually begins                                        }
begin
  case iLoopGameStart of
    1: begin
         iScore := 0;
         Vid_Show_Panel_Score(iScore, true);
         bControllers := true;
         bControllersShoot := true;

         Timer_Shot.Interval := rGameLevels[iMode].iShotSpeed;
         Timer_Enemy_Attack.Interval := rGameLevels[iMode].iEnemySpeed[1];
         Timer_Enemy_Move.Interval := Timer_Enemy_Attack.Interval;

         Timer_Enemy_Helper.Interval := (Random(10) + 5) * 200;
         Timer_Enemy_Helper.Enabled := true;

         Timer_Game_Start.Interval := 400;
       end;
    2: begin
         Gam_Generate_Enemy;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_HitTimer(Sender: TObject);
{ enemy is hit, score is awarded then game reprises                            }
begin
  case iLoopHit of
    1: begin Vid_Show_All_Enemy(false); Snd_Play(pfs_enemy); Timer_Hit.Interval := 175; end;
    2: begin Vid_Show_Enemy(1);         Snd_Play(pfs_shot);  Timer_Hit.Interval := 250; end;
    3: begin Vid_Show_All_Enemy(false); Snd_Play(pfs_enemy); end;
    4: begin Vid_Show_Enemy(1);         Snd_Play(pfs_shot);  end;
    5: begin Vid_Show_All_Enemy(false); Snd_Play(pfs_enemy); Timer_Hit.Interval := Timer_Enemy_Attack.Interval * 2; end;
    6: begin
         Gam_Generate_Enemy;

         Inc(iScore, 10);
         Vid_Show_Panel_Score(iScore, true);

         bControllers := true;
         bControllersShoot := true;

         Timer_Enemy_Helper.Interval := (Random(10) + 5) * 200;
         Timer_Enemy_Helper.Enabled := true;

         Timer_Hit.Enabled := false;

         Gam_Calculate_Speed; // must stay at the END!
       end;
  end; // case..of

  Inc(iLoopHit);
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ game continues after "miss"                                                  }
  var
    i: integer;
begin
  case iLoopMiss of
    1: begin aIm_miss[2].Visible := false; aIm_miss[3].Visible := true; Timer_Miss.Interval := 480; end;
    2:       aIm_miss[4].Visible := true;
    3: begin aIm_miss[4].Visible := false; aIm_miss[5].Visible := true; end;
    4:                                     aIm_miss[5].Visible := false;
    5:       aIm_miss[4].Visible := true;
    6: begin aIm_miss[4].Visible := false; aIm_miss[5].Visible := true; end;
    7:                                     aIm_miss[5].Visible := false;
    8:       aIm_miss[4].Visible := true;
    9: begin aIm_miss[4].Visible := false; aIm_miss[5].Visible := true; Timer_Miss.Interval := 700; end;
   10: begin
         Inc(iMiss); Vid_Show_Panel_Miss(iMiss);

         aIm_miss[2].Visible := true;
         aIm_miss[3].Visible := false;
         aIm_miss[5].Visible := false;

         if iMiss < 3 then                                                      // reprise or game over?
           begin                                                                    // reprise
             Gam_Generate_Enemy;

             for i := 1 to 3 do
               begin
                 aSavemeValue[i] := 3;
                 Vid_Show_Saveme(i);
               end;

             Timer_Enemy_Helper.Interval := (Random(10) + 5) * 200;
             Timer_Enemy_Helper.Enabled := true;

             bControllers := true;
             bControllersShoot := true;
           end
         else
           begin                                                                    // Game Over
             if bSoundIngame then
               begin
                 Snd_Play(pfs_gameover);
                 Timer_Game_Over.Interval := 2500;
               end
             else
               Timer_Game_Over.Interval := 100;

             iLoopGameOver := 1;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  if iLoopMiss < 10 then
    Snd_Play(pfs_miss);

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_ShotTimer(Sender: TObject);
{ player shot: action stops for a while, then reprises or bonus routine begins }
begin
  case iLoopShot of
    1: begin
         aIm_shot[iShotPos, 1].Visible := false;

         if iHelperPos = iShotPos then
           begin
             bControllersShoot := true;

             Timer_Shot.Enabled := false;
           end;
       end;
    2: begin aIm_shot[iShotPos, 2].Visible := false; aIm_shot[iShotPos, 3].Visible := true; end;
    3: begin aIm_shot[iShotPos, 3].Visible := false; aIm_shot[iShotPos, 4].Visible := true; end;
    4: begin
         aIm_shot[iShotPos, 4].Visible := false;

         if iShotPos = iEnemyPos then                                           // collisions
           begin
             Gam_Stop_All_Timers;
             bControllers := false;

             iLoopHit := 1;
             Timer_Hit.Interval := 50;
             Timer_Hit.Enabled := true;
           end
         else
           bControllersShoot := true;

         Timer_Shot.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopShot);
end;





procedure TForm1.Timer_StartupTimer(Sender: TObject);
{ goes into "Time 1" mode , 30 seconds after game is turned on                 }
begin
  if iMode = 0 then
    begin
      iMode := 1;
      Gam_Set_Mode;
    end;

  Timer_Startup.Enabled := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
